## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for students and related components.
"""
import sys
import grok

from datetime import datetime
from zope.formlib.widget import CustomWidgetFactory
from zope.formlib.form import setUpEditWidgets
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.traversing.browser import absoluteURL
from zope.component import (
    createObject,)

from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from reportlab.pdfgen import canvas
from reportlab.lib.units import cm
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import (Frame, Paragraph, Image,
    Table, Spacer)
from reportlab.platypus.tables import TableStyle

from waeup.sirp.accesscodes import invalidate_accesscode, get_access_code
from waeup.sirp.accesscodes.workflow import USED
from waeup.sirp.browser import (
    WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage, WAeUPDisplayFormPage)
from waeup.sirp.browser.breadcrumbs import Breadcrumb
from waeup.sirp.browser.layout import NullValidator
from waeup.sirp.browser.pages import add_local_role, del_local_roles
from waeup.sirp.browser.resources import datepicker, tabs, datatable
from waeup.sirp.browser.viewlets import (
    ManageActionButton, PrimaryNavTab, LeftSidebarLink,
    AddActionButton, ActionButton,
    )
from waeup.sirp.image.browser.widget import (
    ThumbnailWidget, EncodingImageFileWidget,
    )
from waeup.sirp.image.image import createWAeUPImageFile
from waeup.sirp.interfaces import IWAeUPObject, ILocalRolesAssignable
from waeup.sirp.permissions import get_users_with_local_roles
from waeup.sirp.university.interfaces import ICertificate
from waeup.sirp.widgets.datewidget import (
    FriendlyDateWidget, FriendlyDateDisplayWidget)
from waeup.sirp.widgets.restwidget import ReSTDisplayWidget
from waeup.sirp.widgets.objectwidget import (
    WAeUPObjectWidget, WAeUPObjectDisplayWidget)
from waeup.sirp.widgets.multilistwidget import (
    MultiListWidget, MultiListDisplayWidget)
from waeup.sirp.students.interfaces import (
    IStudentsContainer, IStudent, IStudentClearance,
    IStudentPersonal, IStudentBase
    )
from waeup.sirp.students.student import (
    Student,
    )
from waeup.sirp.students.catalog import search

class StudentsTab(PrimaryNavTab):
    """Students tab in primary navigation.
    """

    grok.context(IWAeUPObject)
    grok.order(3)
    grok.require('waeup.viewStudents')
    grok.template('primarynavtab')

    pnav = 4
    tab_title = u'Students'

    @property
    def link_target(self):
        return self.view.application_url('students')

class StudentsBreadcrumb(Breadcrumb):
    """A breadcrumb for the students container.
    """
    grok.context(IStudentsContainer)
    title = u'Students'

class StudentsContainerPage(WAeUPPage):
    """The standard view for student containers.
    """
    grok.context(IStudentsContainer)
    grok.name('index')
    grok.require('waeup.viewStudents')
    grok.template('studentscontainerpage')

    @property
    def title(self):
        return "Students"

    @property
    def label(self):
        return self.title

    def update(self, *args, **kw):
        datatable.need()
        form = self.request.form
        self.hitlist = []
        if 'searchterm' in form and form['searchterm']:
            self.searchterm = form['searchterm']
            self.searchtype = form['searchtype']
        elif 'old_searchterm' in form:
            self.searchterm = form['old_searchterm']
            self.searchtype = form['old_searchtype']
        else:
            if 'search' in form:
                self.flash('Empty search string.')
            return
        self.hitlist = search(query=self.searchterm,
            searchtype=self.searchtype, view=self)
        if not self.hitlist:
            self.flash('No student found.')
        return

class StudentsContainerManageActionButton(ManageActionButton):
    grok.order(1)
    grok.context(IStudentsContainer)
    grok.view(StudentsContainerPage)
    grok.require('waeup.manageStudents')
    text = 'Manage students'


class StudentsContainerManagePage(WAeUPPage):
    """The manage page for student containers.
    """
    grok.context(IStudentsContainer)
    grok.name('manage')
    grok.require('waeup.manageStudents')
    grok.template('studentscontainermanagepage')

    @property
    def title(self):
        return "Students"

    @property
    def label(self):
        return self.title

    def update(self, *args, **kw):
        datatable.need()
        form = self.request.form
        self.hitlist = []
        if 'searchterm' in form and form['searchterm']:
            self.searchterm = form['searchterm']
            self.searchtype = form['searchtype']
        elif 'old_searchterm' in form:
            self.searchterm = form['old_searchterm']
            self.searchtype = form['old_searchtype']
        else:
            if 'search' in form:
                self.flash('Empty search string.')
            return
        #import pdb; pdb.set_trace()
        if not 'entries' in form:
            self.hitlist = search(query=self.searchterm,
                searchtype=self.searchtype, view=self)
            if not self.hitlist:
                self.flash('No student found.')
            return
        entries = form['entries']
        if isinstance(entries, basestring):
            entries = [entries]
        deleted = []
        for entry in entries:
            if 'remove' in form:
                del self.context[entry]
                deleted.append(entry)
        self.hitlist = search(query=self.searchterm,
            searchtype=self.searchtype, view=self)
        if len(deleted):
            self.flash('Successfully removed: %s' % ', '.join(deleted))
        return

class StudentsContainerAddActionButton(AddActionButton):
    grok.order(1)
    grok.context(IStudentsContainer)
    grok.view(StudentsContainerManagePage)
    grok.require('waeup.manageStudents')
    text = 'Add student'
    target = 'addstudent'

class StudentAddFormPage(WAeUPAddFormPage):
    """Add-form to add a student.
    """
    grok.context(IStudentsContainer)
    grok.require('waeup.manageStudents')
    grok.name('addstudent')
    grok.template('studentaddpage')
    form_fields = grok.AutoFields(IStudent)
    title = 'Students'
    label = 'Add student'

    @grok.action('Create student record')
    def addStudent(self, **data):
        student_id = self.request.form.get('form.student_id')
        student = createObject(u'waeup.Student')
        self.applyData(student, **data)
        #import pdb; pdb.set_trace()
        try:
            self.context[student_id] = student
        except KeyError:
            self.flash('The student id chosen already exists.')
            return
        self.flash('Student record created.')
        self.redirect(self.url(self.context[student_id], 'index'))
        return

class StudentBaseDisplayFormPage(WAeUPDisplayFormPage):
    """ Page to display student base data
    """
    grok.context(IStudent)
    grok.name('index')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudentBase)

    @property
    def title(self):
        return 'Student: %s' % self.context.name

    @property
    def label(self):
        return '%s: Base Data' % self.context.name

class StudentBaseManageActionButton(ManageActionButton):
    grok.order(1)
    grok.context(IStudent)
    grok.view(StudentBaseDisplayFormPage)
    grok.require('waeup.manageStudents')
    text = 'Edit base data'
    target = 'edit_base'

class StudentClearanceViewActionButton(ActionButton):
    grok.order(2)
    icon = 'actionicon_view.png'
    grok.context(IStudent)
    grok.view(StudentBaseDisplayFormPage)
    grok.require('waeup.viewStudents')
    text = 'View clearance data'
    target = 'view_clearance'

class StudentPersonalViewActionButton(ActionButton):
    grok.order(3)
    icon = 'actionicon_view.png'
    grok.context(IStudent)
    grok.view(StudentBaseDisplayFormPage)
    grok.require('waeup.viewStudents')
    text = 'View personal data'
    target = 'view_personal'

class StudentBaseManageFormPage(WAeUPEditFormPage):
    """ View to edit student base data
    """
    grok.context(IStudent)
    grok.name('edit_base')
    grok.require('waeup.manageStudents')
    form_fields = grok.AutoFields(IStudentBase).omit('student_id')
    #grok.template('editstudentbase')
    label = 'Edit base data'

    @property
    def title(self):
        return 'Student: %s' % self.context.name

class StudentClearanceDisplayFormPage(WAeUPDisplayFormPage):
    """ Page to display student clearance data
    """
    grok.context(IStudent)
    grok.name('view_clearance')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudentClearance)

    @property
    def title(self):
        return 'Student: %s' % self.context.name

    @property
    def label(self):
        return '%s: Clearance Data' % self.context.name

class StudentClearanceManageActionButton(ManageActionButton):
    grok.order(1)
    grok.context(IStudent)
    grok.view(StudentClearanceDisplayFormPage)
    grok.require('waeup.manageStudents')
    text = 'Edit clearance data'
    target = 'edit_clearance'

class StudentClearanceManageFormPage(WAeUPEditFormPage):
    """ Page to edit student clearance data
    """
    grok.context(IStudent)
    grok.name('edit_clearance')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudentClearance)
    label = 'Edit clearance data'

    @property
    def title(self):
        return 'Student: %s' % self.context.name

class StudentPersonalDisplayFormPage(WAeUPDisplayFormPage):
    """ Page to display student personal data
    """
    grok.context(IStudent)
    grok.name('view_personal')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudentPersonal)

    @property
    def title(self):
        return 'Student: %s' % self.context.name

    @property
    def label(self):
        return '%s: Personal Data' % self.context.name

class StudentPersonalManageActionButton(ManageActionButton):
    grok.order(1)
    grok.context(IStudent)
    grok.view(StudentPersonalDisplayFormPage)
    grok.require('waeup.manageStudents')
    text = 'Edit personal data'
    target = 'edit_personal'

class StudentPersonalManageFormPage(WAeUPEditFormPage):
    """ Page to edit student clearance data
    """
    grok.context(IStudent)
    grok.name('edit_personal')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudentPersonal)
    label = 'Edit personal data'

    @property
    def title(self):
        return 'Student: %s' % self.context.name